"""Simple water flow example using ANUGA

Water flowing down a channel with a topography that varies with time
"""

#------------------------------------------------------------------------------
# Import necessary modules
#------------------------------------------------------------------------------
from anuga.abstract_2d_finite_volumes.mesh_factory import rectangular_cross
from anuga.shallow_water import Domain
from anuga.shallow_water import Reflective_boundary
from anuga.shallow_water import Dirichlet_boundary
from anuga.shallow_water import Time_boundary

#------------------------------------------------------------------------------
# Setup computational domain
#------------------------------------------------------------------------------
length = 40.
width = 5.
dx = dy = 1 #.1           # Resolution: Length of subdivisions on both axes

points, vertices, boundary = rectangular_cross(int(length/dx), int(width/dy),
                                               len1=length, len2=width)
domain = Domain(points, vertices, boundary)
domain.set_name('channel3')                  # Output name
print domain.statistics()

#------------------------------------------------------------------------------
# Setup initial conditions
#------------------------------------------------------------------------------
def topography(x,y):
    """Complex topography defined by a function of vectors x and y."""

    z = -x/100

    N = len(x)
    for i in range(N):
        # Step
        if 10 < x[i] < 12:
            z[i] += 0.4 - 0.05*y[i]

        # Constriction
        #if 27 < x[i] < 29 and y[i] > 3:
        #    z[i] += 2

        # Pole
        #if (x[i] - 34)**2 + (y[i] - 2)**2 < 0.4**2:
        #    z[i] += 2

    return z

def pole_increment(x,y):
    """This provides a small increment to a pole located mid stream
    For use with variable elevation data
    """
    
    z = 0.0*x

    N = len(x)
    for i in range(N):
        # Pole
        if (x[i] - 20)**2 + (y[i] - 2)**2 < 0.4**2:
            z[i] += 0.05
    return z
    
    
domain.set_quantity('elevation', topography)           # elevation is a function
domain.set_quantity('friction', 0.01)                  # Constant friction
domain.set_quantity('stage', expression='elevation')   # Dry initial condition

#------------------------------------------------------------------------------
# Setup boundary conditions
#------------------------------------------------------------------------------
Bi = Dirichlet_boundary([0.4, 0, 0])          # Inflow
Br = Reflective_boundary(domain)              # Solid reflective wall
Bo = Dirichlet_boundary([-5, 0, 0])           # Outflow

domain.set_boundary({'left': Bi, 'right': Bo, 'top': Br, 'bottom': Br})

#------------------------------------------------------------------------------
# Evolve system through time
#------------------------------------------------------------------------------

growing = False
shrinking = False
done = False
for t in domain.evolve(yieldstep=0.1, finaltime=30.0):
    print domain.timestepping_statistics()

    #w = domain.get_quantity('stage').\
    #    get_values(interpolation_points=[[18, 2.5]])
    #print 'Level at gauge point = %.2fm' % w
           
    z = domain.get_quantity('elevation').\
        get_values(interpolation_points=[[20, 2]])           
    print 'Elevation at pole location = %.2fm' % z           

    # Start variable elevation after 10 seconds    
    if t > 10 and not (shrinking or growing or done):
        growing = True
           
    # Turn around when pole has reached a height of 2 m
    if z >= 2 and growing:
        growing = False
        shrinking = True
        
    # Stop changing when pole has shrunk to 1 m
    if z <= 1 and shrinking:
        done = True
        shrinking = growing = False

    # Grow or shrink               
    if growing:       
        domain.add_quantity('elevation', pole_increment)
        
    if shrinking:    
        domain.add_quantity('elevation', lambda x,y: -pole_increment(x,y))    
        
